## -*- coding: utf-8 -*-

from builtins import str
import datetime
import logging
import json
import requests
import sqlalchemy as sa

from pyramid.security import Allow
from pyramid.security import Everyone
from pyramid.threadlocal import get_current_registry 
from ringo.lib.sql import DBSession
from ringo.model import Base
from ringo.model.base import BaseItem, BaseFactory

log = logging.getLogger(__name__)


class EccourseFactory(BaseFactory):

    def create(self, user=None, values=None):
        new_item = BaseFactory.create(self, user, values)
        return new_item


class Eccourse(BaseItem, Base):
    """Course modul. Please define 'participants' as backref in relation
    of items referings to this course."""

    __tablename__ = 'eccourses'
    """Name of the table in the database for this modul. Do not
    change!"""
    _modul_id = None
    """Will be set dynamically. See include me of this modul"""

    # Define columns of the table in the database
    id = sa.Column(sa.Integer, primary_key=True)

    number = sa.Column('number', sa.String, server_default='', unique=True)
    """Unique course identification number"""
    name = sa.Column('name', sa.String, server_default='')
    """A Short description of the course"""
    number_optional = sa.Column('number_optional', sa.String, nullable=False, server_default='')
    size = sa.Column('size', sa.Integer)
    active = sa.Column('active', sa.Integer)
    organisation = sa.Column('organisation', sa.Integer)
    location = sa.Column('location', sa.String, nullable=False, server_default='')
    federal_state = sa.Column('federal_state', sa.Integer)
    plz = sa.Column('plz', sa.String, nullable=False, server_default='')
    region_category = sa.Column('region_category', sa.Integer)
    nuts3 = sa.Column('nuts3', sa.String, nullable=False, server_default='')

    start_date = sa.Column('start_date', sa.Date)
    """start date of the course"""
    end_date = sa.Column('end_date', sa.Date)
    """end date of the course"""
    description = sa.Column('description', sa.Text, server_default='')

    lecturer_firstname = sa.Column('lecturer_firstname', sa.String, nullable=False, server_default='')
    """Currently not used. See lastname"""
    lecturer_lastname = sa.Column('lecturer_lastname', sa.String, nullable=False, server_default='')
    """Lastname ist used for the fullname of the lectures. Even if there are multiple lecturer."""
    lecturer_email = sa.Column('lecturer_email', sa.String, nullable=False, server_default='')
    lecturer_phone = sa.Column('lecturer_phone', sa.String, nullable=False, server_default='')

    @classmethod
    def _get_permissions(cls, modul, item, request):
        """Foo"""

        # Default ACL. Direct access
        permissions = BaseItem._get_permissions(modul, item, request)
        if item is None or not isinstance(item, BaseItem):
            return permissions

        # Custom ACL. Allow everyone to read the courses
        permissions.append((Allow, Everyone, 'read'))
        return permissions

    @property
    def organisation_name(self):
        """Used for filtering the courselisting based on the selection
        organisation (usergroup) in in the application"""
        x = self.get_value("organisation", expand=True)
        return x.lower()

    @property
    def is_available(self):
        """Helper method. Will return True if

        * The current date is not after the enddate 
        * The course is active

        :returns: True or False
        """
        today = datetime.date.today()
        result = (self.active == 1
                  and (not self.end_date or (today <= self.end_date)))
        return result

    @property
    def is_available_str(self):
        """Return a string representation of the availability, used for overviews"""
        return 'Ja' if self.is_available else 'Nein'
    
    @property
    def occupancy(self):
        """Gets the occupancy of the course from an ecimporter instance"""
        registry = get_current_registry()
        settings = registry.settings
        host = settings.get("bewec.importer.host")
        if not host:
            return u'Aktuell keine Reservierungsinformation verfügbar'
        endpoint = '{}/course?id={}'.format(host, self.id)
        hostcert = registry.settings.get("bewec.importer.hostcert")
        clientcert = registry.settings.get("bewec.importer.clientcert")

        # Import here since not all eccourse extensions are installed along a
        # BEWEC system 
        out_bewec = u'–'
        from bewec.model.application import BewecApplication
        result = DBSession.query(sa.func.count(BewecApplication.id)) \
            .filter(BewecApplication.course_id == self.id)
        if result.first():
            out_bewec = result.one()[0]

        out_ec = u'–'
        try:
            response = requests.get(endpoint, data=[], cert=clientcert, verify=hostcert)
            if response.status_code != 200:
                log.warn('Call to ecimporter failed with status code {}'.format(response.status_code))
            else:
                result = json.loads(response.content)
                if result['status'] == 'success' and result['data'].get(str(self.id)):
                    out_ec = result['data'][str(self.id)]['count']
        except requests.ConnectionError as e:
            log.error('Exception while contacting ecimporter: {}'.format(e))

        return u'{1} ({0}/{2})'.format(out_ec, out_bewec, self.size or u'–')

    @classmethod
    def get_item_factory(cls):
        return EccourseFactory(cls)
